@@ -24,6 +24,7 @@
 #define CFRUNLOOP_NEW_API 1
 
 #include <CoreFoundation/CFMachPort.h>
+#include <CoreFoundation/CFPriv.h>
 //#include <IOKit/hid/IOHIDLib.h>
 //#include <unistd.h>
 
@@ -61,9 +62,9 @@ __END_DECLS
 
 #define allChecks() do {	    \
     connectCheck();		    \
+    seizeCheck();                   \
     openCheck();		    \
     terminatedCheck();              \
-    seizeCheck();                   \
 } while (0)
 
 #ifndef max
@@ -76,10 +77,6 @@ __END_DECLS
     ((a < b) ? a:b)
 #endif
 
-typedef struct MyPrivateData {
-    io_object_t			notification;
-    IOHIDDeviceClass *		self;
-} MyPrivateData;
 
 IOCFPlugInInterface ** IOHIDDeviceClass::alloc()
 {
@@ -107,8 +104,11 @@ IOHIDDeviceClass::IOHIDDeviceClass()
     fIsLUNZero			= false;
     fIsTerminated		= false;
     fIsSeized			= false;
+    fAsyncPortSetupDone = false;
 
     fRunLoop 			= NULL;
+    fCFSource			= NULL;
+	fNotifyCFSource		= NULL;
     fQueues			= NULL;
     fDeviceElements		= NULL;
     fReportHandlerQueue		= NULL; 
@@ -123,6 +123,12 @@ IOHIDDeviceClass::IOHIDDeviceClass()
     
     fElementCount 		= 0;
     fElements 			= NULL;
+	
+	fCurrentValuesMappedMemory  = NULL;
+	fCurrentValuesMappedMemorySize = NULL;
+
+	fAsyncPrivateDataRef	= NULL;
+	fNotifyPrivateDataRef   = NULL;
     
     fReportHandlerElementCount	= 0;
     fReportHandlerElements	= NULL;
@@ -141,7 +147,7 @@ IOHIDDeviceClass::~IOHIDDeviceClass()
     }
     
     if (fReportHandlerQueue){
-        (*fReportHandlerQueue)->Release(fReportHandlerQueue);
+        delete fReportHandlerQueue;
         fReportHandlerQueue = 0;
     }
     
@@ -156,24 +162,36 @@ IOHIDDeviceClass::~IOHIDDeviceClass()
         
     if (fDeviceElements)
         CFRelease(fDeviceElements);
+	
+	if (fNotifyCFSource && fRunLoop)
+		CFRunLoopRemoveSource(fRunLoop, fNotifyCFSource, kCFRunLoopDefaultMode);
         
     if (fNotifyPort)
         IONotificationPortDestroy(fNotifyPort);
         
     if (fAsyncPort)
-        mach_port_deallocate(mach_task_self(), fAsyncPort);
+        IONotificationPortDestroy(fAsyncPort);
 
+	if (fAsyncPrivateDataRef) {
+		IOObjectRelease(fAsyncPrivateDataRef->notification);
+		free(fAsyncPrivateDataRef);
+	}
+		
+	if (fNotifyPrivateDataRef) {
+		IOObjectRelease(fNotifyPrivateDataRef->notification);
+		free(fNotifyPrivateDataRef);
+	}
 }
 
-HRESULT	IOHIDDeviceClass::attachQueue (IOHIDQueueClass * iohidQueue)
+HRESULT	IOHIDDeviceClass::attachQueue (IOHIDQueueClass * iohidQueue, bool reportHandler)
 {
     HRESULT res = S_OK;
     
     iohidQueue->setOwningDevice(this);
 
     //  todo add to list
-    if ( fQueues || 
-        ( fQueues = CFSetCreateMutable(kCFAllocatorDefault, 0, 0) ) )
+    if ( !reportHandler && ( fQueues || 
+        ( fQueues = CFSetCreateMutable(kCFAllocatorDefault, 0, 0) ) ) )
     {    
         CFSetAddValue(fQueues, (void *)iohidQueue);
     }
@@ -219,16 +237,23 @@ HRESULT IOHIDDeviceClass::detachOutputTransaction (IOHIDOutputTransactionClass *
     return res;
 }
 
+IOHIDQueueClass * IOHIDDeviceClass::createQueue(bool reportHandler)
+{
+    IOHIDQueueClass * newQueue = new IOHIDQueueClass;
+    
+    // attach the queue to us
+    attachQueue (newQueue, reportHandler);
+
+	return newQueue;
+}
+
 HRESULT IOHIDDeviceClass::queryInterfaceQueue (void **ppv)
 {
     HRESULT res = S_OK;
     
     // create the queue class
-    IOHIDQueueClass * newQueue = new IOHIDQueueClass;
-    
-    // attach the queue to us
-    attachQueue (newQueue);
-    
+    IOHIDQueueClass * newQueue = createQueue();
+
     // add a ref for the one we return
 //    newQueue->addRef();
     
@@ -301,9 +326,7 @@ start(CFDictionaryRef propertyTable, io_service_t inService)
     IOReturn 			res;
     kern_return_t 		kr;
     mach_port_t 		masterPort;
-    CFRunLoopSourceRef		runLoopSource;
     CFMutableDictionaryRef	properties;
-    MyPrivateData		*privateDataRef = NULL;
 
     fService = inService;
     IOObjectRetain(fService);
@@ -320,15 +343,15 @@ start(CFDictionaryRef propertyTable, io_service_t inService)
         return kIOReturnError;
     
     fNotifyPort = IONotificationPortCreate(masterPort);
-    runLoopSource = IONotificationPortGetRunLoopSource(fNotifyPort);
+    fNotifyCFSource = IONotificationPortGetRunLoopSource(fNotifyPort);
     
-    fRunLoop = CFRunLoopGetCurrent();
-    CFRunLoopAddSource(fRunLoop, runLoopSource, kCFRunLoopDefaultMode);
+    fRunLoop = CFRunLoopGetMain();//CFRunLoopGetCurrent();
+    CFRunLoopAddSource(fRunLoop, fNotifyCFSource, kCFRunLoopDefaultMode);
 
-    privateDataRef = (MyPrivateData *)malloc(sizeof(MyPrivateData));
-    bzero(privateDataRef, sizeof(MyPrivateData));
+    fNotifyPrivateDataRef = (MyPrivateData *)malloc(sizeof(MyPrivateData));
+    bzero(fNotifyPrivateDataRef, sizeof(MyPrivateData));
     
-    privateDataRef->self = this;
+    fNotifyPrivateDataRef->self		= this;
 
     // Register for an interest notification of this device being removed. Use a reference to our
     // private data as the refCon which will be passed to the notification callback.
@@ -336,8 +359,8 @@ start(CFDictionaryRef propertyTable, io_service_t inService)
                                             fService,
                                             kIOGeneralInterest,
                                             IOHIDDeviceClass::_deviceNotification,
-                                            privateDataRef,
-                                            &(privateDataRef->notification));
+                                            fNotifyPrivateDataRef,
+                                            &(fNotifyPrivateDataRef->notification));
 
     // Now done with the master_port
     mach_port_deallocate(mach_task_self(), masterPort);
@@ -363,6 +386,11 @@ start(CFDictionaryRef propertyTable, io_service_t inService)
     return kIOReturnSuccess;
 }
 
+// RY: There are 2 General Interest notification event sources.
+// One is operating on the main run loop via fNotifyPort and is internal 
+// to the IOHIDDeviceClass.  The other is used by the client to get removal 
+// notification via fAsyncPort.  This method is used by both and disguished 
+// by the port in the refcon.
 void IOHIDDeviceClass::_deviceNotification( void *refCon,
                                             io_service_t service,
                                             natural_t messageType,
@@ -383,9 +411,12 @@ void IOHIDDeviceClass::_deviceNotification( void *refCon,
     switch(messageType)
     {
         case kIOMessageServiceIsTerminated:
-                
+				
             self->fIsTerminated = true;
             
+			if ( privateDataRef != self->fAsyncPrivateDataRef)
+				break;
+            
             if (self->fRemovalCallback)
             {            
                 ((IOHIDCallbackFunction)self->fRemovalCallback)(
@@ -396,29 +427,40 @@ void IOHIDDeviceClass::_deviceNotification( void *refCon,
             }
             // Free up the notificaiton
             IOObjectRelease(privateDataRef->notification);
-            free(privateDataRef);
+            free(self->fAsyncPrivateDataRef);
+			self->fAsyncPrivateDataRef = 0;
             break;
         
         case kIOMessageServiceIsRequestingClose:
-            if ((options == kIOHIDOptionsTypeSeizeDevice) &&
+			if ( privateDataRef != self->fNotifyPrivateDataRef)
+				break;
+
+            if ((options & kIOHIDOptionsTypeSeizeDevice) &&
                 (options != self->fCachedFlags))
             {
-                self->fIsSeized = true;
                 self->stopAllQueues(true);
                 if (self->fReportHandlerQueue)
-                    (*self->fReportHandlerQueue)->stop(self->fReportHandlerQueue);
+                    self->fReportHandlerQueue->stop();
+				self->close();
+                self->fIsSeized = true;
             }
             break;
             
         case kIOMessageServiceWasClosed:
+			if ( privateDataRef != self->fNotifyPrivateDataRef)
+				break;
+
             if (self->fIsSeized &&
-                (options == kIOHIDOptionsTypeSeizeDevice) &&
+                (options & kIOHIDOptionsTypeSeizeDevice) &&
                 (options != self->fCachedFlags))
             {
                 self->fIsSeized = false;
+
+				self->open(self->fCachedFlags);
+
                 self->startAllQueues(true);
                 if (self->fReportHandlerQueue)
-                    (*self->fReportHandlerQueue)->start(self->fReportHandlerQueue);
+                    self->fReportHandlerQueue->start();
             }
             break;
     }    
@@ -444,8 +486,8 @@ createAsyncEventSource(CFRunLoopSourceRef *source)
     context.release = NULL;
     context.copyDescription = NULL;
 
-    cfPort = CFMachPortCreateWithPort(NULL, fAsyncPort,
-                (CFMachPortCallBack) IODispatchCalloutFromMessage,
+    cfPort = CFMachPortCreateWithPort(NULL, IONotificationPortGetMachPort(fAsyncPort),
+                (CFMachPortCallBack) _cfmachPortCallback,
                 &context, &shouldFreeInfo);
     if (!cfPort)
         return kIOReturnNoMemory;
@@ -468,39 +510,58 @@ CFRunLoopSourceRef IOHIDDeviceClass::getAsyncEventSource()
 
 IOReturn IOHIDDeviceClass::createAsyncPort(mach_port_t *port)
 {
-    IOReturn ret;
+    IOReturn		ret;
+    mach_port_t		masterPort;
 
     connectCheck();
     
     // If we already have a port, don't create a new one.
     if (fAsyncPort) {
         if (port)
-            *port = fAsyncPort;
+            *port = IONotificationPortGetMachPort(fAsyncPort);
         return kIOReturnSuccess;
     }
 
-    ret = IOCreateReceivePort(kOSAsyncCompleteMessageID, &fAsyncPort);
-    if (kIOReturnSuccess == ret) {
+    // First create a master_port for my task
+    ret = IOMasterPort(MACH_PORT_NULL, &masterPort);
+    if (ret || !masterPort)
+        return kIOReturnError;
+
+	fAsyncPort = IONotificationPortCreate(masterPort);
+	
+    if (fAsyncPort) {
         if (port)
-            *port = fAsyncPort;
+            *port = IONotificationPortGetMachPort(fAsyncPort);
 
         if (fIsOpen) {
-            natural_t asyncRef[1];
-            mach_msg_type_number_t len = 0;
-        
-            // async kIOCDBUserClientSetAsyncPort,  kIOUCScalarIScalarO,    0,	0
-            return io_async_method_structureI_structureO(
-                    fConnection, fAsyncPort, asyncRef, 1,
-                    kIOHIDLibUserClientSetAsyncPort, NULL, 0, NULL, &len);
+			ret = finishAsyncPortSetup();
         }
     }
+    
+	mach_port_deallocate(mach_task_self(), masterPort);
+    masterPort = 0;
 
     return ret;
 }
 
+IOReturn IOHIDDeviceClass::finishAsyncPortSetup()
+{
+	natural_t				asyncRef[1];
+	mach_msg_type_number_t  len = 0;
+			
+	finishReportHandlerQueueSetup();
+
+    fAsyncPortSetupDone = true;
+
+	// async kIOCDBUserClientSetAsyncPort,  kIOUCScalarIScalarO,    0,	0
+	return io_async_method_scalarI_scalarO(
+			fConnection, IONotificationPortGetMachPort(fAsyncPort), asyncRef, 1,
+			kIOHIDLibUserClientSetAsyncPort, NULL, 0, NULL, &len);
+}
+
 mach_port_t IOHIDDeviceClass::getAsyncPort()
 {
-    return fAsyncPort;
+    return IONotificationPortGetMachPort(fAsyncPort);
 }
 
 IOReturn IOHIDDeviceClass::open(UInt32 flags)
@@ -530,36 +591,33 @@ IOReturn IOHIDDeviceClass::open(UInt32 flags)
     fIsOpen = true;
     fIsSeized = false;
 
-    if (fAsyncPort) {
-        natural_t asyncRef[1];
-        mach_msg_type_number_t len = 0;
-    
-        // async 
-        // kIOHIDLibUserClientSetAsyncPort,  kIOUCScalarIScalarO,    0,	0
-        ret = io_async_method_scalarI_scalarO(
-                fConnection, fAsyncPort, asyncRef, 1,
-                kIOHIDLibUserClientSetAsyncPort, NULL, 0, NULL, &len);
-        if (ret != kIOReturnSuccess) {
+    if (!fAsyncPortSetupDone && fAsyncPort) {
+		ret = finishAsyncPortSetup();
+
+		if (ret != kIOReturnSuccess) {
             close();
             return ret;
         }
     }
     
     // get the shared memory
-    vm_address_t address = nil;
-    vm_size_t size = 0;
-    
-    ret = IOConnectMapMemory (	fConnection, 
-                                IOHIDLibUserClientElementValuesType, 
-                                mach_task_self(), 
-                                &address, 
-                                &size, 
-                                kIOMapAnywhere	);
-    if (ret == kIOReturnSuccess)
-    {
-        fCurrentValuesMappedMemory = address;
-        fCurrentValuesMappedMemorySize = size;
-    }
+	if ( !fCurrentValuesMappedMemory )
+	{
+		vm_address_t address = nil;
+		vm_size_t size = 0;
+		
+		ret = IOConnectMapMemory (	fConnection, 
+									IOHIDLibUserClientElementValuesType, 
+									mach_task_self(), 
+									&address, 
+									&size, 
+									kIOMapAnywhere	);
+		if (ret == kIOReturnSuccess)
+		{
+			fCurrentValuesMappedMemory = address;
+			fCurrentValuesMappedMemorySize = size;
+		}
+	}
     
     return ret;
 }
@@ -605,12 +663,38 @@ IOReturn IOHIDDeviceClass::setRemovalCallback(
                                    IOHIDCallbackFunction 	removalCallback,
                                    void *			removalTarget,
                                    void *			removalRefcon)
-{
-    fRemovalCallback	= removalCallback;
-    fRemovalTarget	= removalTarget;
-    fRemovalRefcon	= removalRefcon;
+{	
+	IOReturn ret = kIOReturnSuccess;
 
-    return kIOReturnSuccess;
+	if (!fAsyncPrivateDataRef)
+	{
+		
+		fAsyncPrivateDataRef = (MyPrivateData *)malloc(sizeof(MyPrivateData));
+		bzero(fAsyncPrivateDataRef, sizeof(MyPrivateData));
+		
+		fAsyncPrivateDataRef->self		= this;
+
+        if (!fAsyncPort) {     
+            ret = createAsyncPort(0);
+            if (kIOReturnSuccess != ret)
+                return ret;
+        }
+
+		// Register for an interest notification of this device being removed. Use a reference to our
+		// private data as the refCon which will be passed to the notification callback.
+		ret = IOServiceAddInterestNotification( fAsyncPort,
+												fService,
+												kIOGeneralInterest,
+												IOHIDDeviceClass::_deviceNotification,
+												fAsyncPrivateDataRef,
+												&(fAsyncPrivateDataRef->notification));
+	}
+	
+    fRemovalCallback    = removalCallback;
+    fRemovalTarget      = removalTarget;
+    fRemovalRefcon      = removalRefcon;
+
+    return ret;
 }
 
 IOReturn IOHIDDeviceClass::getElementValue(IOHIDElementCookie	elementCookie,
@@ -649,18 +733,16 @@ IOReturn IOHIDDeviceClass::setElementValue(
     kern_return_t           	kr = kIOReturnBadArgument;
     IOHIDElementStruct		element;
     
+    allChecks();
+
     if (!getElement(elementCookie, &element))
         return kr;
-        
-
 
     // we are only interested feature and output elements
     if ((element.type != kIOHIDElementTypeFeature) && 
             (element.type != kIOHIDElementTypeOutput))
         return kr;
                 
-    allChecks();
-
     // get ptr to shared memory for this element
     if (element.valueLocation < fCurrentValuesMappedMemorySize)
     {
@@ -754,6 +836,8 @@ IOReturn IOHIDDeviceClass::fillElementValue(IOHIDElementCookie		elementCookie,
                                             IOHIDEventStruct *		valueEvent)
 {
     IOHIDElementStruct	element;
+
+    allChecks();
     
     if (!getElement(elementCookie, &element) || (element.type == kIOHIDElementTypeCollection))
         return kIOReturnBadArgument;
@@ -763,9 +847,7 @@ IOReturn IOHIDDeviceClass::fillElementValue(IOHIDElementCookie		elementCookie,
     void *		longValue = 0;
     UInt32		longValueSize = 0;
     UInt64		timestamp = 0;
-    
-    allChecks();
-    
+        
     // get ptr to shared memory for this element
     if (element.valueLocation < fCurrentValuesMappedMemorySize)
     {
@@ -861,7 +943,7 @@ IOHIDDeviceClass::setReport (	IOHIDReportType			reportType,
         asyncRef[kIOAsyncCalloutFuncIndex] = (natural_t) _hidReportCallback;
         asyncRef[kIOAsyncCalloutRefconIndex] = (natural_t) hidRefcon;
     
-        ret = io_async_method_scalarI_scalarO( fConnection, fAsyncPort, asyncRef, kIOAsyncCalloutCount, kIOHIDLibUserClientAsyncSetReport, in, 5, NULL, &len);
+        ret = io_async_method_scalarI_scalarO( fConnection, IONotificationPortGetMachPort(fAsyncPort), asyncRef, kIOAsyncCalloutCount, kIOHIDLibUserClientAsyncSetReport, in, 5, NULL, &len);
     
     }
     else
@@ -941,7 +1023,7 @@ IOHIDDeviceClass::getReport (	IOHIDReportType			reportType,
         asyncRef[kIOAsyncCalloutFuncIndex] = (natural_t) _hidReportCallback;
         asyncRef[kIOAsyncCalloutRefconIndex] = (natural_t) hidRefcon;
     
-        ret = io_async_method_scalarI_scalarO( fConnection, fAsyncPort, asyncRef, kIOAsyncCalloutCount, kIOHIDLibUserClientAsyncGetReport, in, 5, NULL, &len);
+        ret = io_async_method_scalarI_scalarO( fConnection, IONotificationPortGetMachPort(fAsyncPort), asyncRef, kIOAsyncCalloutCount, kIOHIDLibUserClientAsyncGetReport, in, 5, NULL, &len);
     
     }
     else
@@ -1074,7 +1156,6 @@ IOReturn IOHIDDeviceClass::setInterruptReportHandlerCallback(
                                 void * 				callbackRefcon)
                                 
 {
-    CFRunLoopSourceRef	evSource;
     IOReturn		ret = kIOReturnSuccess;
 
     fInputReportCallback 	= callback;
@@ -1086,67 +1167,91 @@ IOReturn IOHIDDeviceClass::setInterruptReportHandlerCallback(
     // Lazy set up of the queue.
     if ( !fReportHandlerQueue )
     {
-	if ( !( fReportHandlerQueue = allocQueue() ) )
-            return kIOReturnError;
-            
-        CFSetRemoveValue(fQueues, (void *)fReportHandlerQueue);
-    
-        ret = (*fReportHandlerQueue)->create(fReportHandlerQueue, 0, 8);
+		fReportHandlerQueue = createQueue(true);
+
+        ret = fReportHandlerQueue->create(0, 8);
         
         if (ret != kIOReturnSuccess)
             goto SET_REPORT_HANDLER_CLEANUP;
         
         for (int i=0; i<fReportHandlerElementCount; i++)
         {
-            ret = (*fReportHandlerQueue)->addElement(fReportHandlerQueue, (IOHIDElementCookie)fReportHandlerElements[i].cookie, 0);
+            ret = fReportHandlerQueue->addElement((IOHIDElementCookie)fReportHandlerElements[i].cookie, 0);
     
             if (ret != kIOReturnSuccess)
                 goto SET_REPORT_HANDLER_CLEANUP;
         }
 
-        ret = (*fReportHandlerQueue)->createAsyncEventSource(fReportHandlerQueue, &evSource);
-        
-        if (ret != kIOReturnSuccess)
-            goto SET_REPORT_HANDLER_CLEANUP;
-        
-        ret = (*fReportHandlerQueue)->setEventCallout(fReportHandlerQueue, _hidReportHandlerCallback, this, 0);
-        
-        if (ret != kIOReturnSuccess)
-            goto SET_REPORT_HANDLER_CLEANUP;
-        
-        CFRunLoopAddSource(CFRunLoopGetCurrent(), evSource, kCFRunLoopDefaultMode);
-        
-        ret = (*fReportHandlerQueue)->start(fReportHandlerQueue);
-
-        if (ret != kIOReturnSuccess)
-            goto SET_REPORT_HANDLER_CLEANUP;
+		
+		if ( fAsyncPort && fIsOpen )
+		{
+			ret = finishReportHandlerQueueSetup();
+			if (ret != kIOReturnSuccess)
+				goto SET_REPORT_HANDLER_CLEANUP;
+		}
     }    
     
     return kIOReturnSuccess;
     
 SET_REPORT_HANDLER_CLEANUP:
-    (*fReportHandlerQueue)->Release(fReportHandlerQueue);
+    delete fReportHandlerQueue;
     fReportHandlerQueue = 0;
     
     return ret;
 }
 
+IOReturn IOHIDDeviceClass::finishReportHandlerQueueSetup()
+{
+	IOReturn ret = kIOReturnError;
+	
+	if ( fReportHandlerQueue )
+	{
+		do {
+			ret = fReportHandlerQueue->setEventCallout(_hidReportHandlerCallback, this, 0);
+			
+			if (ret != kIOReturnSuccess) break;
+				
+			ret = fReportHandlerQueue->setAsyncPort(IONotificationPortGetMachPort(fAsyncPort));
+			
+			if (ret != kIOReturnSuccess) break;
+					
+			ret = fReportHandlerQueue->start();
+
+			if (ret != kIOReturnSuccess) break;
+            			
+		} while ( false );
+	}
+	return ret;
+}
+
+void IOHIDDeviceClass::_cfmachPortCallback(CFMachPortRef cfPort, mach_msg_header_t *msg, CFIndex size, void *info) {
+    mach_msg_header_t *			msgh = (mach_msg_header_t *)msg;
+	IOHIDDeviceClass *			self = (IOHIDDeviceClass *) info;
+	
+	if ( !self )
+		return;
+		
+    if( msgh->msgh_id == kOSNotificationMessageID)
+		IODispatchCalloutFromMessage(cfPort, msg, info);
+	else if ( self->fReportHandlerQueue )
+		IOHIDQueueClass::queueEventSourceCallback(cfPort, msg, size, self->fReportHandlerQueue);
+}
+
 void IOHIDDeviceClass::_hidReportHandlerCallback(void * target, IOReturn result, void * refcon, void * sender)
 {
     IOHIDEventStruct 		event;
     IOHIDElementStruct		element;
     IOHIDDeviceClass *		self = (IOHIDDeviceClass *)target;
-    IOHIDQueueInterface ** 	queue = self->fReportHandlerQueue;
+    IOHIDQueueClass *		queue = self->fReportHandlerQueue;
     AbsoluteTime 		zeroTime = {0,0};
     UInt32			size = 0;
 
     if (!self || !self->fIsOpen)
         return;
             
-    while (result == kIOReturnSuccess) {
-        result = (*queue)->getNextEvent(queue, &event, zeroTime, 0);
+    while ((result = queue->getNextEvent( &event, zeroTime, 0)) == kIOReturnSuccess) {
         
-        if ( !result && (event.longValueSize == 0)) 
+        if ( event.longValueSize == 0)
         {                        
             self->getElement(event.elementCookie, &element);                
             size = element.bytes;
@@ -1156,7 +1261,7 @@ void IOHIDDeviceClass::_hidReportHandlerCallback(void * target, IOReturn result,
             self->convertWordToByte((const UInt32 *)(&(event.value)), (UInt8 *)self->fInputReportBuffer, size << 3);
             
         }
-        else if (!result && event.longValueSize != 0 && (event.longValue != NULL))
+        else if (event.longValueSize != 0 && (event.longValue != NULL))
         {
             size = min(self->fInputReportBufferSize, event.longValueSize);
             bcopy(event.longValue, self->fInputReportBuffer, size);
@@ -1292,7 +1397,7 @@ IOReturn IOHIDDeviceClass::startAllQueues(bool deviceInitiated)
     
         CFSetGetValues(fQueues, (const void **)queues);
         
-        for (int i=0; queues && i<queueCount && ret==kIOReturnSuccess; i++)
+        for (int i=0; queues && i<queueCount; i++)
         {
             ret = queues[i]->start(deviceInitiated);
         }
@@ -1858,11 +1963,11 @@ kern_return_t IOHIDDeviceClass::FindReportHandlers(CFDictionaryRef properties)
 
 IOHIDElementType IOHIDDeviceClass::getElementType(IOHIDElementCookie elementCookie)
 {
-    IOHIDElementType type = (IOHIDElementType) 0;
+    IOHIDElementStruct      element;
+    IOHIDElementType        type = (IOHIDElementType) 0;
     
-    for (long index = 0; index < fElementCount; index++)
-        if (fElements[index].cookie == (unsigned long) elementCookie)
-            type = (IOHIDElementType) fElements[index].type;
+    if (getElement(elementCookie, &element))
+            type = (IOHIDElementType) element.type;
     
     return type;
 }
