@@ -32,6 +32,7 @@
 
 class IOHIDQueueClass;
 
+
 struct IOHIDElementStruct
 {
     unsigned long	cookie;
@@ -49,6 +50,11 @@ typedef struct IOHIDElementStruct IOHIDElementStruct;
 
 class IOHIDDeviceClass : public IOHIDIUnknown
 {
+typedef struct MyPrivateData {
+    io_object_t				notification;
+    IOHIDDeviceClass *		self;
+} MyPrivateData;
+
 private:
     // friends with queue class
     friend class IOHIDQueueClass;
@@ -68,15 +74,20 @@ protected:
     struct InterfaceMap 	fHIDDevice;
     io_service_t 		fService;
     io_connect_t 		fConnection;
-    mach_port_t 		fAsyncPort;
+    IONotificationPortRef 		fAsyncPort;
     CFRunLoopRef 		fRunLoop;
     IONotificationPortRef 	fNotifyPort;
     CFRunLoopSourceRef 		fCFSource;
+	CFRunLoopSourceRef		fNotifyCFSource;
     bool 			fIsOpen;
     bool 			fIsLUNZero;
     bool			fIsTerminated;
     bool			fIsSeized;
+    bool            fAsyncPortSetupDone;
     UInt32			fCachedFlags;
+	
+	MyPrivateData * fAsyncPrivateDataRef;
+	MyPrivateData * fNotifyPrivateDataRef;
     
     IOHIDCallbackFunction 	fRemovalCallback;
     void *			fRemovalTarget;
@@ -97,7 +108,7 @@ protected:
     long 				fReportHandlerElementCount;
     IOHIDElementStruct * 		fReportHandlerElements;
     
-    IOHIDQueueInterface **		fReportHandlerQueue;
+    IOHIDQueueClass *		fReportHandlerQueue;
     
     IOHIDReportCallbackFunction		fInputReportCallback;
     void *				fInputReportTarget;
@@ -121,7 +132,14 @@ protected:
                         UInt8 *        dst,
                         UInt32         bitsToCopy);
                         
+	IOReturn finishAsyncPortSetup();
+	IOReturn finishReportHandlerQueueSetup();
+	
+	IOHIDQueueClass * createQueue(bool reportHandler=false);
+
     // Call back methods
+    static void _cfmachPortCallback(CFMachPortRef cfPort, mach_msg_header_t *msg, CFIndex size, void *info);
+
     static void _hidReportCallback(void *refcon, IOReturn result, UInt32 bufferSize);
     
     static void _deviceNotification( void *refCon,
@@ -133,7 +151,7 @@ protected:
                            
 public:
     // add/remove a queue 
-    HRESULT attachQueue (IOHIDQueueClass * iohidQueue);
+    HRESULT attachQueue (IOHIDQueueClass * iohidQueue, bool reportHandler = false);
     HRESULT detachQueue (IOHIDQueueClass * iohidQueue);
     
     // add/remove a queue 
@@ -146,6 +164,7 @@ public:
     IOHIDElementType getElementType (IOHIDElementCookie elementCookie);
     UInt32 getElementByteSize (IOHIDElementCookie elementCookie);
 
+
     // IOCFPlugin stuff
     static IOCFPlugInInterface **alloc();
 
